package com.autentia.citas.managers;

import java.util.Calendar;
import java.util.List;
import java.util.concurrent.Future;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import com.autentia.citas.dao.Dao;
import com.autentia.citas.model.Cita;
import com.autentia.citas.model.Contacto;
import com.autentia.citas.utils.Utils;

public class CitasManagerTest {
	
	
	@BeforeClass
	public static void initTests() throws Exception {		
		if(Utils.container==null) {
			Utils.startContainer();
		}
	}
	
	@SuppressWarnings("unchecked")
	@AfterClass
	public static void releaseTests() throws Exception {	
		// Limpiamos la Base de datos para no interferir en otros tests
		Dao dao =(Dao)Utils.ctx.lookup("java:global/citas/JPADaoImpl");
		List<Contacto> contactos = dao.getAll(Contacto.class); 
		for(Contacto c:contactos) {
			dao.delete(c);
		}

	//	container.close();
	}
	
	@Test
	public void createCitas() throws Exception {		// 
		// Usamos el contexto JNDI para buscar el EJB...Y usando el nombre estandarizado...
		CitasManager manager =(CitasManager)Utils.ctx.lookup("java:global/citas/CitasManager");
		
		Cita cita = new Cita();		
			cita.setAsunto("Quedada en el parque con los nios");
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.YEAR, 2014);
			cal.set(Calendar.DAY_OF_MONTH, 21);
			cal.set(Calendar.MONTH, 7);
			cita.setFecha(cal.getTime());		
			
		Contacto contacto = new Contacto();
			contacto.setNombre("Francisco Javier");
			contacto.setApellidos("Martnez Pez");
			contacto.addEmail("fjmpaez@autentia.com");
			contacto.addEmail("fjmpaez@acme.com");
			contacto.addTelefono("915551111");
			contacto.addTelefono("620999999");
			
		manager.citarme(contacto, cita);
		
		Assert.assertTrue(manager.getAllCitas().size()==1);
		Assert.assertTrue(manager.getAllContactos().size()==1);
		Assert.assertTrue(manager.getCitasWith(contacto).size()==1);
		
		
		// Probamos cita con contacto ya existente
		cal.set(Calendar.YEAR, 2012);
		cal.set(Calendar.DAY_OF_MONTH, 21);
		cal.set(Calendar.MONTH, 7);		
	
		Cita cita2 = new Cita();		
			cita2.setAsunto("Reunin de seguimiento proyecto ACME LA CAMA");
			cita2.setFecha(cal.getTime());
		
		manager.citarme(contacto, cita2);
		
		Assert.assertTrue(manager.getAllCitas().size()==2);
		Assert.assertTrue(manager.getAllContactos().size()==1);
		Assert.assertTrue(manager.getCitasWith(contacto).size()==2);
	}
	
	@Test(timeout=9000)
	public void recuerda1() throws Exception {
		CitasManager manager =(CitasManager)Utils.ctx.lookup("java:global/citas/CitasManager");
		manager.recuerdaleLaCita(null);
	}
	
	@Test
	public void recuerda2() throws Exception {
		CitasManager manager =(CitasManager)Utils.ctx.lookup("java:global/citas/CitasManager");
		// A mi esto del futuro me da miedo. LO ha debido disear Doc el de Regreso Al futuro
		long time1 = System.currentTimeMillis();
		Future<Boolean> futuro = manager.recuerdaleLaCitaAck(null);
		long time2 = System.currentTimeMillis();
		Assert.assertTrue((time2-time1)< 9000 );
		
		// Ahora se bloquea esperando respuesta.		
		Boolean result = futuro.get();
		
		long time3 = System.currentTimeMillis();
		Assert.assertTrue((time3-time1)>= 10000 );
		Assert.assertTrue(result);		
		// Hay tambin una versin con espera activa: futuro.get(timeout, unit)		
	}
	
	
}
